import styled from "styled-components";
import CustomOverlay from "../../src/CustomOverlay";
import TestIcon from "./TestIcon";

import ApartmentIcon from "./ApartmentIcon";
import HouseIcon from "./HouseIcon";
import BuildingIcon from "./BuildingIcon";
import StoreIcon from "./StoreIcon";
import PlantIcon from "./PlantIcon";
import MultipleHouseIcon from "./MultipleHouseIcon";
import React, { useState, FunctionComponent, useMemo } from "react";

interface ITestMarkerProps {
  className?: string;
  position: {
    lat: number,
    lng: number,
  };
  areaSize: number;
  collision: boolean;
  price: number;
  tradeType: TMapRealPriceCategory;
}

/**
 * @enum A: 아파트, O: 오피스텔, S: 단독/다가구, R: 연립/다세대, L: 토지, N: 상업용
 */
export type TMapRealPriceCategory = "A" | "O" | "S" | "R" | "L" | "N";

const TestMarker: FunctionComponent<ITestMarkerProps> = (props) => {
  const content = useMemo(() => document.createElement("div"), []);

  const [hover, setHover] = useState(false);

  function changeRealTradeToColor(tradeType: TMapRealPriceCategory) {
    switch (tradeType) {
      case "A": return "#64c5fd";
      case "S": return "#df99fb";
      case "O": return "#e986a1";
      case "N": return "#fbc299";
      case "L": return "#b8e986";
      case "R": return "#99acfb";
    }
  }

  function changeRealTradeToIcon(tradeType: TMapRealPriceCategory) {
    switch (tradeType) {
      case "A": return <ApartmentIcon />;
      case "S": return <HouseIcon />;
      case "O": return <BuildingIcon />;
      case "N": return <StoreIcon />;
      case "L": return <PlantIcon />;
      case "R": return <MultipleHouseIcon />;
    }
  }

  return (
    <CustomOverlay
      zIndex={hover ? 9999 : props.collision ? 0 : 99}
      position={props.position}
      content={content}
    >
      <div className={props.className} style={{ opacity: props.collision ? 0.7 : 1 }}>
        <div className="content">
          <div className="trade_type" style={{ backgroundColor: changeRealTradeToColor(props.tradeType) }}>
            {changeRealTradeToIcon(props.tradeType)}
          </div>
          <p className="price">{(props.price / props.areaSize).toFixed(0)}/평</p>
        </div>

        <div className="arrow">
          <div className="arrow-wrapper">
            <div className="arrow-inner"></div>
          </div>
        </div>

        <TestIcon color={changeRealTradeToColor(props.tradeType)} />
      </div>
    </CustomOverlay>
  );
};

export default styled(TestMarker)`
  & {
    * {
      margin: 0;
      padding: 0;
      box-sizing: border-box;
    }

    font-size: 0;
    text-align: center;
    position: relative;
    width: 107px;
    height: 43.6px;
    transform: translateY(-50%);

    .content {
      background-color: #fffffd;
      padding: 6px;
      width: 100%;
      height: 37px;
      border-radius: 11px;
      display: flex;
      align-items: center;
      box-shadow: 0 1px 10px 0 rgba(0, 0, 0, 0.06);
      position: relative;
      z-index: 3;

      .trade_type {
        min-width: 25px;
        margin-right: 7px;
        height: 25px;
        border-radius: 8px;
        display: flex;
        justify-content: center;
        align-items: center;
      }

      .price {
        margin-top: 1px;
        font-size: 12px;
        font-weight: bold;
        font-style: normal;
        /* line-height: 10.83; */
        letter-spacing: -0.4px;
        color: rgb(51, 51, 51);
      }
    }

    .arrow {
      position: relative;
      margin-top: -15px;

      .arrow-wrapper {
        width: 23.2px;
        height: 23.2px;
        overflow: hidden;
        padding: 0 2.7px;
        display: inline-block;

        .arrow-inner {
          transform: rotate(135deg);
          background-color: #fffffd;
          width: 18px;
          height: 18px;
          position: relative;
          border-radius: 1px;
        }
      }
    }

    p {
      margin: 0;
      padding: 0;
      box-sizing: border-box;
    }
  }
`;
