import styled from "styled-components";

import TestMarker from "../TestMarker";
import React, { useEffect, useState, useContext, FunctionComponent } from "react";
import MapContext from "../../src/Map/context";

export interface ITestOverlayProps {
  // center: TLocation;
  // level?: number;
  // filter?: boolean;
  // category?: string;
  // bounds?: TBounds;
  data: any[];
  onClick?: (location: any) => void;
}

const TestOverlay: FunctionComponent<ITestOverlayProps> = (props) => {
  const { map } = useContext(MapContext);

  const [markers, setMarkers] = useState<any[]>([]);

  function toMarker(list: any[]): any[] {
    const balloonBounds = [] as [[number, number], [number, number]][];

    function testCollision(b: [[number, number], [number, number]]) {
      for (const balloonBound of balloonBounds) {
        const boundsCondition = [[b[0][0], b[0][1]], [b[1][0], b[0][1]], [b[0][0], b[1][1]], [b[1][0], b[1][1]]]
          .some((point) => [point[0] >= balloonBound[0][0], point[0] < balloonBound[1][0], point[1] <= balloonBound[0][1], point[1] > balloonBound[1][1]].every((v) => v));

        if (boundsCondition) return true;
      }

      return false;
    }

    return list
      .map((d, i) => [d.location.lat, d.location.lng, i])
      .sort()
      .map((v) => {
        const value = list[v[v.length - 1]];
        const latlng = new kakao.maps.LatLng(value.location.lat, value.location.lng);
        const kakaoPoint = map.getProjection().containerPointFromCoords(latlng);

        const point = kakaoPoint.toString().slice(1, -1).split(", ").map((c) => +c) as [number, number];
        const testBounds = [[point[0] - 100, point[1] + 70], [point[0] + 20, point[1] - 10]] as [[number, number], [number, number]];

        const collision = testCollision(testBounds);
        if (collision === false) balloonBounds.push(testBounds);

        const id = `${value.pnu}-${value.category}-${value.trade_type}`;

        return { ...value, collision, id };
      });
  }

  useEffect(() => {
    setMarkers((props.data));
  }, [props.data]);

  return (
    <>
      {markers.map((v) => (
        <TestMarker
          collision={v.collision}
          position={v.location}
          price={v.price}
          areaSize={v.size}
          tradeType={(v.category as any)}
        />
      ))}
    </>
  );
};

export default styled(TestOverlay)``;
