import React, { FC, useState, useEffect } from "react";

import ReactDOM from "react-dom";

import styled from "styled-components";

import KakaoMap from "../src/Map";
import { getTileSetList } from "./tileMapHanlder";
import useLoadTiles from "./test";
import TestMarker from "./TestMarker";
import TestOverlay from "./TestOverlay";

interface ITestComponentProps {
  className?: string;
}

let TestComponent: FC<ITestComponentProps> = (props) => {
  const [container, setContainer] = useState<HTMLDivElement>(null);
  const [mapCenter] = useState({ lat: 37.498851344498604, lng: 127.02777438528088 });

  const { state, action } = useLoadTiles();

  useEffect(() => {
    // 타일셋 생성
    window.kakao.maps.Tileset.add('TILE_NUMBER',
      new window.kakao.maps.Tileset({
        width: 256,
        height: 256,
        getTile: function (x: number, y: number, z: number) {
          const div = document.createElement('div');
          div.innerHTML = x + ', ' + y + ', ' + z;
          div.style.fontSize = '36px';
          div.style.fontWeight = 'bold';
          div.style.lineHeight = '256px'
          div.style.color = '#4D4D4D';
          div.style.textAlign = 'center';
          div.style.border = '1px dashed #FF5050';
          div.style.fontSize = '2.2rem';
          return div;
        },
      }));
  }, []);

  useEffect(() => {
    action.setMarkersStore(state.tileSet, state.markersStore);
  }, [state.tileSet]);

  return (
    <div className={props.className}>
      <div className="main-modal"></div>

      {/* <div ref={(ref) => setContainer(ref)} className="map">
        {container && (
          <KakaoMap
            container={container}
            center={mapCenter}
            level={4}
            onZoomChange={() => {
              action.clearMarkers();
            }}
            customOverlayMapTypes="TILE_NUMBER"
            onTilesLoaded={({ bounds, zoomLevel }) => {
              const sw = new kakao.maps.LatLng(bounds[0].lat, bounds[0].lng);
              const ne = new kakao.maps.LatLng(bounds[1].lat, bounds[1].lng);

              if (zoomLevel < 6) {
                const tiles = getTileSetList(sw, ne, zoomLevel);
                action.setTilesSet(tiles);
              }
            }}
          >
            <TestOverlay data={state.markers} />
          </KakaoMap>
        )}
      </div> */}
    </div>
  );
};

TestComponent = styled(TestComponent)`
  width: 100vw;
  height: 100vh;
  position: relative;
  display: flex;

  .main-modal {
    width: 400px;
    height: 100%;
    background-color: #ff0000;
    position: relative;
    z-index: 100;
  }

  .map {
    width: calc(100% - 400px);
    height: 100%;
    position: relative;
  }

`;

ReactDOM.render(<TestComponent />, document.getElementById("root"))
