import produce from 'immer';
import axios from 'axios';
import React, { useEffect } from 'react';

declare global {
  interface Window {
    kakao: any;
  }
}

export type TTileType = {
  id: number;
  column: number;
  row: number;
  level: number;
};

export type TMarkersStore = {
  id: number,
  marker: any[],
};

interface ITileload {
  state: ITileloadState;
  action: ITileloadAction;
}

interface ITileloadState {
  tileSet: TTileType[];
  markersStore: TMarkersStore[];
  markers: any[];
  apiCount: number;
  test: {
    id: string;
    markerDatas: any[];
  }[];
  loading: boolean;
}

interface ITileloadAction extends TTileloadAction { }
type TTileloadAction = typeof generateAction extends (...args: any[]) => infer R ? R : never;
const generateAction = (update: (recipe: (draft: ITileloadState) => void) => void) => {
  const setTilesSet = (tiles: TTileType[]) => update((draft) => {
    draft.tileSet = tiles;
  });

  const setStore = (id: number, data: any[]) => update((draft) => {
    draft.markersStore.push({ id, marker: data });
  })

  const pushMarker = (marker) => update((draft) => {
    draft.markers.push(...marker)
  })

  const clearMarkers = () => update((draft) => {
    draft.markers = []
  })

  const setMarkersStore = (tiles: TTileType[], markerStore) => update((draft) => {
    draft.loading = true;
    const a = tiles.map((v) => {
      return markerStore.filter((d) => v.id === d.id)[0];
    }).filter((v) => v);
    draft.markers = [(a.map((v) => v.marker) as any).flatMap((v) => v)][0];
    const calledTiles: TTileType[] = [];
    const callTiles: TTileType[] = [];
    // 마커 아이디 배열
    const ids = [] as number[];
    const markers = [] as any[];
    let errorCount = 0;
    let successCount = 0;

    tiles.map((tile) => {
      if (markerStore.filter((v) => v.id === tile.id).length === 0) callTiles.push(tile);
      else calledTiles.push(tile);
    });

    callTiles.forEach((value) => {
      (async () => {
        try {
          const response = await axios.get(`https://api.nazipsa.kr/api/v1/map/markers/real_price/tms/${value.column}/${value.row}/${value.level}`, { timeout: 5000 });

          setStore(value.id, response.data.items);

          markers.push({
            id: value.column * value.row * value.level,
            markerDatas: response.data.items
          });

          successCount += 1;
        } catch (e) {
          errorCount += 1;
        } finally {
          if (successCount === (callTiles.length - errorCount)) {
            pushMarker((markers as any).flatMap((v) => {
              ids.push(v.id);
              return v.markerDatas;
            }));
          }
        }
      })();
    });

    calledTiles.forEach((tile) => {
      pushMarker(markerStore.filter((v) => v.id === tile. id)[0].marker);
    })

    draft.loading = false;
  });

  return {
    setTilesSet,
    setMarkersStore,
    clearMarkers,
  };
};

const initializer = (): ITileloadState => {
  return {
    tileSet: [],
    markersStore: [],
    markers: [],
    apiCount: 0,
    test: [],
    loading: false,
  };
};

const useLoadTiles = (): ITileload => {
  const [state, setState] = React.useState(() => initializer());

  const update = (recipe: (draft: ITileloadState) => void) => setState((prev) => produce(prev, recipe));

  const action = generateAction(update);

  return {
    state,
    action
  };
};

export default useLoadTiles;
