import React, { FC, useState, useEffect } from "react";

import ReactDOM from "react-dom";

import styled from "styled-components";
import KakaoMap from "../../src/Map";
import TestMarker from "../TestMarker";
import { getTileSetList } from "./handler/getTileSetList";
import { tileToMarkerWithTileData } from "./handler/tileToMarkerWithTileData";
import getRealPriceLayer from "./api/getRealPriceLayer";

interface ITestComponentProps {
  className?: string;
}

let TestComponent: FC<ITestComponentProps> = (props) => {
  const [container, setContainer] = useState<HTMLDivElement>(null);
  const [mapCenter] = useState({ lat: 37.498851344498604, lng: 127.02777438528088 });

  const [marker, setMarker] = useState([] as any);

  // 화면에 모든 타일맵을 가져온다.
  // 해당 타일맵 x, y, z를 가져오고 그 좌표 값을 가지고 api를 요청한다.
  // [
  //   {
  //     id: "123,123,1",
  //     markerDatas: [MarkerData, MarkerData],
  //   },
  //   {
  //     id: "123,123,2",
  //     markerDatas: [MarkerData, MarkerData],
  //   }
  // ]
  // 이런식으로 데이터를 넣는다.
  // 타일 맵 개수대로 다 받으면 그때 화면에 보여줘야함.
  // timeout 이 일어나서 못받아올수있는데 그럴떄는 카운트를 세서 length - count 해주기

  useEffect(() => {
    // 타일셋 생성
    window.kakao.maps.Tileset.add('TILE_NUMBER',
      new window.kakao.maps.Tileset({
        width: 256,
        height: 256,
        getTile: function (x: number, y: number, z: number) {
          const div = document.createElement('div');
          div.innerHTML = x + ', ' + y + ', ' + z;
          div.style.fontSize = '36px';
          div.style.fontWeight = 'bold';
          div.style.lineHeight = '256px'
          div.style.color = '#4D4D4D';
          div.style.textAlign = 'center';
          div.style.border = '1px dashed #FF5050';
          div.style.fontSize = '2.2rem';
          return div;
        },
      }));
  }, []);

  return (
    <div className={props.className}>
      <div ref={(ref) => setContainer(ref)} className="map">
        {container && (
          <KakaoMap
            container={container}
            center={mapCenter}
            level={4}
            onZoomChange={() => {
              // action.clearMarkers();
            }}
            customOverlayMapTypes="TILE_NUMBER"
            onTilesLoaded={async ({ bounds, zoomLevel }) => {
              // console.log(bounds);
              const sw = new kakao.maps.LatLng(bounds[0].lat, bounds[0].lng);
              const ne = new kakao.maps.LatLng(bounds[1].lat, bounds[1].lng);

              if (zoomLevel < 6) {
                const tiles = getTileSetList(sw, ne, zoomLevel);

                tiles.forEach((value, index) => {
                  (async () => {
                    try {
                      const response = await getRealPriceLayer(value.column, value.row, value.level, { timeout: 120 });

                      console.log(response);
                      console.log(marker);

                      // setMarker([...marker, response.data.items]);


                      // setStore(callTiles[index].id, response.data.items);

                      // response.data.items.map((marker) => {
                      //   if (ids.includes(marker.id)) return;
                      //   ids.push(marker.id);
                      //   pushMarker(marker);
                      // });
                    } catch (e) {
                      // console.error(e);
                    }
                  })();
                });
                // const test = await tiles.reduce(async(array, value) => {
                //   const a = await getRealPriceLayer(value.column, value.row, value.level, { timeout: 120 });

                //   array.push(a);
                // }, [] as any);
                // const test = await tileToMarkerWithTileData(tiles);
                // console.log(test);
              }
            }}
          >
            {/* {state.markers.map((v, k) => (
              <TestMarker
                position={v.location}
                price={v.price}
                areaSize={v.size}
                tradeType={(v.category as any)}
              />
            ))} */}
          </KakaoMap>
        )}
      </div>
    </div>
  );
};

TestComponent = styled(TestComponent)`
  width: 100vw;
  height: 100vh;
  position: relative;
  display: flex;

  .map {
    width: 100%;
    height: 100%;
    position: relative;
  }
`;

// ReactDOM.render(<TestComponent />, document.getElementById("root"))
