import { TTileType } from "./test";

interface ILatLng {
  // 위도
  Ma: number,
  // 경도
  La: number
}


export function getTile(lat: number, lng: number, zoom: number) {
  // 위도와 경도를 이용하여 그 위치에 존재하는 타일(col, row, level)을 리턴함
  const TILE_SIZE = 256;
  const ZOOM = [0.25, 0.5, 1.0, 2.0, 4.0, 8.0, 16.0, 32.0, 64.0, 128.0, 256.0, 512.0, 1024, 2048]
  const OFFSET_X = -30000
  const OFFSET_Y = -60000
  const RATIO = 2.5

  const latlng = new kakao.maps.LatLng(lat, lng);
  const wcong = latlng.toCoords();
  const wcong_x = wcong.getX();
  const wcong_y = wcong.getY();
  const z = zoom - 1;
  const tile = ZOOM[z] * TILE_SIZE;
  const tx = ((wcong_x / RATIO) - OFFSET_X) / tile;
  const ty = ((wcong_y / RATIO) - OFFSET_Y) / tile;

  return {
    tx: Math.floor(tx),
    ty: Math.floor(ty),
    zoom
  }
}

export function getTileSetList(southWestLatLng: ILatLng, northEastLatLng: ILatLng, level: number) {
  // 바운드 박스의 남서쪽의 타일과 북동쪽의 타일을 이용하여 바운드 안에 있는 모든 타일셋을 반환하는 함수
  const tileSetList: TTileType[] = [];

  const { tx: minX, ty: minY } = getTile(southWestLatLng.Ma, southWestLatLng.La, level);
  const { tx: maxX, ty: maxY } = getTile(northEastLatLng.Ma, northEastLatLng.La, level);

  for (let i = minX; i <= maxX; i++) {
    for (let j = minY; j <= maxY; j++) {
      tileSetList.push({
        id: i * j * level,
        column: i,
        row: j,
        level: level
      });
    }
  }

  return tileSetList;
}
